/** \file
	\note	Built and tested with WinAVR-20090313.
			Tested with ESP8266EX v0.9.2.2 AT Firmware.bin. Quite possibly would not work with any other version.
			Make sure AVR and ESP8266 UART speeds are matching (AT+CIOBAUD). 
			Uses SHT11 temperature + humidity sensor
			and MPXA6115A pressure sensor (connected to ADC input with simple amplifier for better precision).
			POSTs string with all measurements data to php file that parses it and stores into database every 5 minutes.
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>
#include <stdlib.h>
#include <util/delay.h>
#include "uart.h"
#include "dbg_putchar.h"
#define LOG_ENABLED
#include "log.h"

#include "board.h"
#include "ADC.h"
#include "SHTxx.h"
#include "ESP8266.h"
#include "systick.h"
#include "config.h"

#define UART_BAUD_RATE      19200  

char szTelnetBuffer[35];


static uint8_t reconnect(void)
{
	LOG("Reconnecting...\n");
	char cmd[64];
	snprintf(cmd, sizeof(cmd), "AT+CWJAP=\"%s\",\"%s\"\r\n", SSID, PASS);
	return ESP8266_send_cmd(cmd, 1);
}

   
int main (void)
{
    /* Read and clear reset flags */
    unsigned char reset_flags = MCUSR;                // Save reset flags.
    MCUSR       = 0;                    // Clear all flags previously set.

	dbg_tx_init();	
	fdevopen (dbg_putchar, 0);

	MSG("ESP8266 web thermometer\n");
	MSG("Built: %s, %s\n", __DATE__, __TIME__);
	MSG("MCUSR = %d\n", reset_flags);


    init_beeper();
    init_LED1();
    init_LED2();
    Led1On();
    Led2On();
    beep();
    Led1Off();
    Led2Off();
	

    /*
     *  Initialize UART library, pass baudrate and AVR cpu clock
     *  with the macro 
     *  UART_BAUD_SELECT() (normal speed mode )
     *  or 
     *  UART_BAUD_SELECT_DOUBLE_SPEED() ( double speed mode)
     */
    uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) ); 
    
	// 10 ms tick period
	systick_init(10);
	
    /*
     * now enable interrupt, since UART library is interrupt controlled
     */
    sei();


    uint8_t error = 0;
    value humid_val,temp_val;
    char buf[20], bufPressure[10];

	ADC_init();
	#define ADC_BUF_SIZE 10
	uint16_t ADCbuf[ADC_BUF_SIZE];
	uint8_t ADCptr = 0;
	for (uint8_t i=0; i<ADC_BUF_SIZE; i++)
		ADCbuf[i] = ADC_read();
	uint32_t ADCsum;
	uint16_t ADCavg;


	ESP8266_send_cmd("AT+RST\r\n", 1);	

	// give time for startup and automatic connect
	for (uint16_t i=0; i<1000; i++)
	{
		_delay_ms(10);
	}
	
	s_connectionreset();
	error = 0;
	unsigned char checksum;
	//WARNING! s_measure may take up to 2s (max timeout), so you'll probably
	//have to rewrite this function into non-blocking code if your application does sth time-critical
	error+=s_measure((unsigned char*) &temp_val.i,&checksum,TEMP);
	error+=s_measure((unsigned char*) &humid_val.i,&checksum,HUMID);	
	

    while (1)
    {
        systick_reset();
		
		for (uint8_t temp=0;temp<200;temp++)
        {
            if (temp%4==0)
                Led1Toggle();
            _delay_loop_2(0xFFFF);
        }
		
		ADCbuf[ADCptr++] = ADC_read();
		ADCptr = ADCptr % ADC_BUF_SIZE;
		ADCsum = 0;
		for (uint8_t i=0; i<ADC_BUF_SIZE; i++)
			ADCsum += ADCbuf[i];
		ADCavg = ADCsum/ADC_BUF_SIZE;
		LOG("Pressure: %d ADC\r\n", ADCavg);
		float fPressure;
		fPressure = 0.0887 * ADCavg + 963.3;	//coefficient from LMS method calibration, individual calibration needed!
		itoa((int)(fPressure + 0.5), bufPressure, 10);		
		
        s_connectionreset();
        error = 0;
        unsigned char checksum;
        //WARNING! s_measure may take up to 2s (max timeout), so you'll probably
        //have to rewrite this function into non-blocking code if your application does sth time-critical
        error+=s_measure((unsigned char*) &temp_val.i,&checksum,TEMP);
        error+=s_measure((unsigned char*) &humid_val.i,&checksum,HUMID);
        if (error != 0)
        {
            s_connectionreset();
            cli();
            strcpy(szTelnetBuffer, "MEAS ERROR\r\n");
            sei();
        }
        else
        {
            humid_val.f=(float)humid_val.i;
            temp_val.f=(float)temp_val.i;
            calc_sht11(&humid_val.f,&temp_val.f);
            temp_val.i=(int32_t)(temp_val.f*10);

            cli();
            strcpy(szTelnetBuffer, "Temp: ");
            if (temp_val.i < 0)
                strcat(szTelnetBuffer, "-");
            temp_val.i = abs(temp_val.i);
            itoa(temp_val.i/10, buf, 10);
            strcat(szTelnetBuffer, buf);
            strcat(szTelnetBuffer,".");
            itoa(temp_val.i%10, buf, 10);
            strcat(szTelnetBuffer,buf);
            strcat(szTelnetBuffer,"C  ");

            humid_val.i = (int32_t)(humid_val.f);
            strcat(szTelnetBuffer,"RH: ");
            itoa(humid_val.i, buf, 10);
            strcat(szTelnetBuffer,buf);
            strcat(szTelnetBuffer,"%25  ");		// % sign encoded
			strcat(szTelnetBuffer,bufPressure);
			strcat(szTelnetBuffer,"hPa");
            sei();
        }
		
		// disable echo
		ESP8266_send_cmd("ATE0\r\n", 1);		
		
		// check connection
		uint8_t connected = 0;
		ESP8266_send_cmd("AT+CWJAP?\r\n", 0);
		if (ESP8266_recv(SSID, NULL) == 0)
		{
			connected = 1;
		}
		if (!connected)
		{
			if (reconnect() == 0)
			{
				connected = 1;
			}
		}
		
		if (connected)
		{
			int err;
			char cmd[256];
			char cipsend[32];
			// single connection mode
			ESP8266_send_cmd("AT+CIPMUX=0\r\n", 1);		
			snprintf(cmd, sizeof(cmd), "AT+CIPSTART=\"TCP\",\"%s\",%s\r\n",
				HTTP_IP, HTTP_PORT);
			cmd[sizeof(cmd)-1] = '\0';
			ESP8266_send_cmd(cmd, 1);
			err = ESP8266_recv("Linked", NULL);
			if (!err)
			{
				static PROGMEM char HTTP_POST[] = {
					"POST /"HTTP_TARGET" HTTP/1.0\r\n"\
					"Host: "HTTP_HOST"\r\n"
					"Content-type: application/x-www-form-urlencoded\r\n"
					"Content-length: %d\r\n\r\n"
					"s=%s"
				};
				snprintf_P(cmd, sizeof(cmd), HTTP_POST,
					strlen(szTelnetBuffer)+2,
					szTelnetBuffer
					);
				cmd[sizeof(cmd)-1] = '\0';
					
				snprintf(cipsend, sizeof(cipsend), "AT+CIPSEND=%d\r\n", strlen(cmd));
				cipsend[sizeof(cipsend)-1] = '\0';
				err = ESP8266_send_cmd(cipsend, 0);
				ESP8266_recv(">", NULL);
				ESP8266_send_cmd(cmd, 0);
				err = ESP8266_recv("SEND OK", NULL);
				if (err)
				{
					LOG("Failed to send data\n");
				}
				ESP8266_send_cmd("AT+CIPCLOSE\r\n", 0);
				ESP8266_recv("OK", "ERROR");				
			}
		}
		
		// pause
		LOG("Pausing...\n");
		while(systick_val() < 30000);
    }
    return (1);
}

